{% extends "testable.jinja" %}
{% block imports %}
if 'transformer' not in globals():
    from mage_ai.data_preparation.decorators import transformer
{% endblock %}

{% block content %}
@transformer
def transform(*args, **kwargs):
    """
    Template code for a transformer block.

    Add more parameters to this function if this block has multiple parent blocks.
    There should be one parameter for each output variable from each parent block.

    Args:
        args: The input variables from upstream blocks

    Returns:
        Anything (e.g. data frame, dictionary, array, int, str, etc.)
    """
    # Specify your transformation logic here
{{ code }}
    return {}
{% endblock %}
