{% extends "testable.jinja" %}
{% block imports %}
if 'data_loader' not in globals():
    from mage_ai.data_preparation.decorators import data_loader
{% endblock %}

{% block content %}
@data_loader
def load_data(*args, **kwargs):
    """
    Template code for loading data from any source.

    Returns:
        Anything (e.g. data frame, dictionary, array, int, str, etc.)
    """
    # Specify your data loading logic here
{{ code }}
    return {}
{% endblock %}
